/*  Author: Vasyl Penuyk (Lab_U55)
*  Date: March, 2018	Ukraine.
*
*  This library is free software; you can redistribute it and/or
*  modify it under the terms of the GNU Lesser General Public
*  License as published by the Free Software Foundation; either
*  version 2.1 of the License, or (at your option) any later version.
*
*  This library is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*  Lesser General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public
*  License along with this library; if not, write to the Free Software
*  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
*	TM1637W 9.x (Wiring version)
*  These libraries are intended for the TM1637- driver 7-segment 6-bit display 
*	with the common anode and 16 buttons used.
*  Attention, the TM1637 does not support the simultaneous pressing of several 
*	buttons at the hardware level.
*
*  Modified records:
*	Ver 9.01, Date: April, 2018
*	Corrected TWI protocol, ASCII table moved in to FlashMemory
*    
*  Ver 9.02, Date: 15 May, 2018
*	Fixed function dispStr_TM1637.
*
*	Ver 9.03, Date: 18 May, 2018
*	Added function keyscan.
*
*	Ver 9.04, Date: 31 July, 2018
*	Update functions dispStr_TM1637(), added dispCombiData().
*
*/
//******************************************************************************************//

#include "TM1637LabUW.h"
#if defined(ARDUINO) && ARDUINO >= 100
#include <Arduino.h>
#else
#include <WProgram.h>
#endif

const byte table[96]PROGMEM = { // ASCII to 7segment data
    0x00, 0x86, 0x22, 0x49, 0x56, 0x6B, 0x53, 0x46, 0x61, 0x43,
    0x63, 0x46, 0x80, 0x40, 0x80, 0x30, 0x3F, 0x06, 0x5B, 0x4F,
    0x66, 0x6D, 0x7D, 0x07, 0x7F, 0x6F, 0x09, 0x58, 0x60, 0x48,
    0x42, 0xA7, 0x37, 0x77, 0x7C, 0x39, 0x5E, 0x79, 0x71, 0x3D,
    0x76, 0x30, 0x0E, 0x75, 0x38, 0x55, 0x54, 0x3F, 0x73, 0x67,
    0x50, 0x6D, 0x78, 0x3E, 0x2A, 0x6A, 0x76, 0x6E, 0x5B, 0x61,
    0x24, 0x43, 0x23, 0x08, 0x20, 0x5F, 0x7C, 0x58, 0x5E, 0x7B,
    0x71, 0x7B, 0x74, 0x18, 0x0E, 0x75, 0x38, 0x55, 0x54, 0x5C,
    0x73, 0x67, 0x50, 0x6D, 0x78, 0x1C, 0x2A, 0x6A, 0x76, 0x6E,
    0x5B, 0x46, 0x57, 0x70, 0x1D                            };

TM1637LabUW::TM1637LabUW(byte clockPin, byte dataPin)
	{
		pin_CLK= clockPin;
		pin_DIO= dataPin;
	}
//******************************************************************************************/

void TM1637LabUW::init_TM1637()
	{
		CLK_HIGH; CLK_WRITE;
		DIO_HIGH; DIO_WRITE;
		setBright_TM1637(0);
	}
//******************************************************************************************/

void TM1637LabUW::clear_TM1637(void)
	{	
		start_TM1637();
		wrByte_TM1637(0x40);
		stop_TM1637();
		start_TM1637();
		wrByte_TM1637(0xC0);
		for (byte i= 0; i< 6; ++i)
			{       			
       			wrByte_TM1637(0x00);
       		}
		stop_TM1637();
    }
//******************************************************************************************/

void TM1637LabUW::start_TM1637(void)
// Встановлюється низький рівень DIO при високому рівні CLK
    {
		DIO_HIGH;
		CLK_HIGH;
		SH_DEL;
		DIO_LOW;
    }
//******************************************************************************************/

void TM1637LabUW::stop_TM1637(void)
// Встановлюється високий рівень DIO при високому рівні CLK
    {
		CLK_LOW;
		SH_DEL;
		DIO_LOW;
		SH_DEL;
		CLK_HIGH;
		SH_DEL;
		DIO_HIGH;
		LO_DEL;								// Затримка між сесіями (according to Datasheet)
    }
//******************************************************************************************/

void TM1637LabUW::wrByte_TM1637(byte oneByte)
	{
		byte count_ACK= 0;
		for (byte i=0; i< 8; i++)
			{
				CLK_LOW;
				SH_DEL;
				if (bitRead(oneByte, i)) DIO_HIGH;
				else DIO_LOW;
				SH_DEL;
				CLK_HIGH;
				SH_DEL;
			}
//	}
//	void TM1637LabUW::ack_TM1637()
//	ACK- підпорядкований пристрій утримує низький рівень шини DIO впродовж одного такту CLK, або більше
//	при перевищенні частоти- до опрацювання інформації, у вийняткових ситуаціях не видає сигнал.
//	{
    	CLK_LOW;  
		DIO_LOW;
		SH_DEL;
    	DIO_HIGH;
    	SH_DEL;
    	while(DIO_READ)
      		{
        		count_ACK++;
        		SH_DEL;
        		if (200< count_ACK)
        			{
          				// (go to restart/stop, any more actions);
          				// спробуєм ще раз, чи достатньо повідомлення про помилку?
          				break;
        			}
      		}
      	DIO_LOW;
      	CLK_HIGH;
      	SH_DEL;
	}
/*  For diagnostic getting ACK
		CLK_LOW;  
		DIO_LOW;
		SH_DEL;
    	DIO_HIGH;
    	MD_DEL;
    	CLK_HIGH;
    	SH_DEL;
    	while(DIO_READ)
      		{
        		count_ACK++;
        		//SH_DEL;
        		if (200<count_ACK)
        			{
          				// (goto restart/stop any more actions);
          				break;
        			}
      		}
    } 		
*/
//******************************************************************************************/

void TM1637LabUW::dispStr_TM1637(String dispStr, byte attachDigits)
//byte[] TM1637LabUW::parseStr_TM1637(String dataStr)
	{
    	byte data7seg[7]={0,0,0,0,0,0};
    	byte j= 6;
	    byte divPos= 0;     // Divider positions data (dots)
		for (int k= dispStr.length(); k>= 0; k--)
			{
        		unsigned char symbol= dispStr.charAt(k);
        		if(symbol== '.')
			     	{
		        		divPos|= 1<<j;
            			continue;
          			}
        		if (symbol > 0x20) symbol -= 0x20;
        		else symbol = 0x00;
        		data7seg[j] = pgm_read_byte(&(table[symbol]))^(0x80*(bitRead(divPos,j)));
        		if (j) j--;
        		else break;
        	}
/*		return data7seg[]; 		// &*- and need return divPos with &*divPos ;
	}
void TM1637LabUW::dispArray_TM1637(byte data7seg[], byte attachDigits)
    {
*/			
		byte diffDigs= 6- attachDigits; 					//difference of digits
		start_TM1637();
		wrByte_TM1637(0x40);
		stop_TM1637();
		start_TM1637();
		wrByte_TM1637(0xC0);
		for (byte i= 0; i< 6; ++i)
			{
       			if (i>= attachDigits) wrByte_TM1637(0x00); // Clear used't places
       			else wrByte_TM1637(data7seg[i+ diffDigs]);
       		}
		stop_TM1637();
	}
//******************************************************************************************/

void TM1637LabUW::dispLeds_TM1637(byte dataLeds, byte position)
// Для індикації окремими світлодіодами. byte position- розряд виведення 0-mostLeft; 6-mostRight;
// It is established when using single LEDs instead of the 7-segment display only.
	{	
		start_TM1637();
		wrByte_TM1637(0x44);							// Data Set Command 0x40 + Fixed Address Mode 0x04
		stop_TM1637();
		start_TM1637();
		wrByte_TM1637(0xC0 + position);					// Data Set Address 0xC0 + Address (0x00 to 0x05)
		wrByte_TM1637(dataLeds);
       	stop_TM1637();
    }
//******************************************************************************************/

void TM1637LabUW::dispCombiData_TM1637(String dataStr, byte ledsGroup1, byte ledsGroup2)
// displaying Combined Data: symbols at first (4-places Left Position), single leds to 16 at the end Right Position
// відображення комбінованих даних (символи не більше 4 розрядів спочатку, до 16 роздільних світлодіодів наприкінці)
	{
		byte data7seg[5]={0,0,0,0};
    	byte j= 4;
	    byte divPos= 0;     								// Divider positions data (dots)
		for (int k= dataStr.length(); k>= 0; k--)
			{
        		unsigned char symbol= dataStr.charAt(k);	// 6126	Flash, 284 SRAM bytes
        		//unsigned char symbol= dataStr[k];			// 6136 Flash, 285 SRAM bytes
			    if(symbol== '.')
			     	{
		        		//bitSet(divPos, j);				// 6166 Flash, 284 SRAM bytes
		        		divPos|= 1<<j;						// 6126 Flash, 284 SRAM bytes
            			continue;
          			}
        		if (symbol > 0x20) symbol -= 0x20;
        		else symbol = 0x00;
        		/*
        		data7seg[j] = (bitRead(divPos,j))? (pgm_read_byte(&(table[symbol]))|0x80) : pgm_read_byte(&(table[symbol])); // 6130 Flash, 284 SRAM bytes
        		
        		if (bitRead(divPos,j)) data7seg[j] = pgm_read_byte(&(table[symbol]))|0x80;
        		else data7seg[j] = pgm_read_byte(&(table[symbol]));							// 6138 Flash, 284 SRAM bytes
        		*/			
        		data7seg[j]= pgm_read_byte(&(table[symbol]))^(0x80*(bitRead(divPos,j)));		// 6126	Flash, 284 SRAM bytes
				
        		if (j) j--;
        		else break;
		    }
		start_TM1637();
		wrByte_TM1637(0x40);							// Data Set Command 0x40
		stop_TM1637();
		start_TM1637();
		wrByte_TM1637(0xC0);							// Data Set Address 0xC0
		for(byte n= 0; n< 6; n++)
			{
				if(n< 4) wrByte_TM1637(data7seg[n]);	// Write arrayData
       			else if (n == 4) wrByte_TM1637(ledsGroup1);
       			else wrByte_TM1637(ledsGroup2);
       		}
		stop_TM1637();
	}

//******************************************************************************************/

void TM1637LabUW::setBright_TM1637(byte brightness)
	{
		byte intensity= 0x88 + brightness;
		start_TM1637();
		wrByte_TM1637(intensity);
		stop_TM1637();
	}
//******************************************************************************************/

byte TM1637LabUW::scanKeys_TM1637(void)
	{
		byte rekey= 0x00;
		start_TM1637();
		wrByte_TM1637(0x42);
		DIO_HIGH;
		for (byte j = 0; j < 8; j++)
			{
				CLK_LOW;
				LO_DEL;
        		CLK_HIGH;
        		if(DIO_READ) bitClear(rekey, j);
        		else bitSet(rekey, j);
        		LO_DEL;
        	}
        DIO_LOW;
		SH_DEL;
		DIO_HIGH; 						// Sending NACK.
		CLK_HIGH;
		SH_DEL;
		CLK_LOW;
		stop_TM1637();
		if(rekey)rekey-=7;
		return rekey; 
    }