/* Author: Vasyl Penuyk (Lab_U55)
*  Date: March, 2018	Ukraine.
*
*  This library is free software; you can redistribute it and/or
*  modify it under the terms of the GNU Lesser General Public
*  License as published by the Free Software Foundation; either
*  version 2.1 of the License, or (at your option) any later version.
*
*  This library is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*  Lesser General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public
*  License along with this library; if not, write to the Free Software
*  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
*	TM1637W 9.x (Wiring version)
*  These libraries are intended for the TM1637- driver 7-segment 6-bit display 
*	with the common anode and 16 buttons used.
*  Attention, the TM1637 does not support the simultaneous pressing of several 
*	buttons at the hardware level.
*
*  Modified records:
*	Ver 9.01, Date: April, 2018
*	Corrected TWI protocol, ASCII table moved in to FlashMemory
*    
*  Ver 9.02, Date: 15 May, 2018
*	Fixed function dispStr_TM1637.
*
*	Ver 9.03, Date: 18 May, 2018
*	Added function keyscan.
*
*	Ver 9.04, Date: 31 July, 2018
*	Update functions dispStr_TM1637(), added dispCombiData().
*
*/
/*******************************************************************************/

#ifndef TM1637LabUW_h
#define TM1637LabUW_h
//#include <inttypes.h>
//#include <Arduino.h>

#if defined(ARDUINO) && ARDUINO >= 100
#include <Arduino.h>
#else
#include <WProgram.h>
#endif

#define DIO_HIGH pinMode(pin_DIO, INPUT)			// External pullup
#define DIO_LOW pinMode(pin_DIO, OUTPUT)
#define DIO_READ digitalRead(pin_DIO)
#define DIO_WRITE digitalWrite(pin_DIO,LOW)

#define CLK_HIGH pinMode(pin_CLK, INPUT)			// External pullup
#define CLK_LOW pinMode(pin_CLK, OUTPUT)
#define CLK_WRITE digitalWrite(pin_CLK,LOW)

#define SH_DEL delayMicroseconds(2)					// Short delay
#define LO_DEL delayMicroseconds(50)				// Long delay

class TM1637LabUW
	{
		public:
			TM1637LabUW(byte clockPin, byte dataPin);
			void init_TM1637(void);
			void clear_TM1637(void);
			void start_TM1637(void);
			void stop_TM1637(void);
			void wrByte_TM1637(byte oneByte);
			void dispStr_TM1637(String dispStr, byte attachDigits);
			void dispLeds_TM1637(byte dataLeds, byte position);
			void dispCombiData_TM1637(String dataStr, byte ledsGroup1, byte ledsGroup2);
			void setBright_TM1637(byte brightness);
			virtual byte scanKeys_TM1637(void);
		private:
			byte pin_CLK;
			byte pin_DIO;
			byte brightness;
			byte rekey;
	};

#endif